;; Generated package description from auctex.el  -*- no-byte-compile: t -*-
(define-package "auctex" "13.2.3" "Integrated environment for *TeX*" '((emacs "26.1")) :commit "ef2999e6d7c3232c0fe82d1de89ddda17b88c5b0" :maintainer '(nil . "auctex-devel@gnu.org") :keywords '("tex" "latex" "texinfo" "context" "doctex" "preview-latex") :url "https://www.gnu.org/software/auctex/")
